% Program: VL15.m 27.03.15 15:30 creates 3D movies in fgA.ps, fgB.ps 
%         (first erase fgA, fgB),  finally shows magnetization curves.
% Computes the ideal periodic Vortex Lattice from Ginzburg-Landau
% theory as described in E.H.Brandt, Phys.Rev.B 68 (2003) 54506, 1-11.
% The order parameter om = omega(x,y) =|psi|^2 and the magnetic field
% are expressed by finite Fourier series for the triangular or square
% lattice, tabulated on a grid of vectors x,y, of size nx,ny within a
% rectangular basis cell -.5 <x< +.5, 0 <y< 0.5*y2, y2=sqrt(3)/2 for
% triangular lattice. The Fourier coefficients ak, hk of om(x,y) and
% B(x,y) are then calculated by iterating equations for them, starting
% with the ak=ak0 from Abrikosov's Bc2 solution, and with hk=0.
% During this iteration the profiles of om(x,0) and H(x,0) are shown
% like a movie. Unit length is the vortex spacing x1=1, depends on b.
%
% The result depends on two parameters: the GL parameter kap=kappa 
% (taken from a table of lkap values in akap) and the average 
% induction <B(x,y)> = b*Bc2 here called b0, divided by the upper 
% critical field Bc2=b0/b, tabulated in the array ab of length lb.
% This computation is is done within two loops, for all b and kap.  
% After the computation, the magnetization curves are plotted. 
%      (From: 28.11.06 FLLtri3c.m CLEANED, 13.2.07 VL13.m)
%
  colordef black; format compact; hold off; close all; 
  scs=get(0,'screensize'); figure('position',scs);  % show full screen
  more off; colormap white; % no color in mesh;  for MATLAB 6.5 or 7.3
                 p1=1.0; % p>0 =lengths in sec of pauses at some plots
%                   
% akap=[.75 .85 1 1.2 1.5 2 3 5 7 10 14];      % list of kappa values
% ab=[.998 .95 .9:-.1:.2 .15 10.^(-[4:11]/4)]; % list of b=B/Bc2 values
% ab=[.998 .975:-.025:.125 10.^(-[8:24]/8) ];  % denser kappa values 
% akap=[.85 1 2 3 5 10], ab=[.8 .4 .2 10.^(-[4:11]/4)]; % fast example
% akap=[.85 1 1.3 2 3 5], ab=[.998 .7 .5 .3 .2 .1 .03 .01 ], % faster 
%%  akap=[2 5 10]; ab=[.3 .1 .03 .01 .003]; p1=1;      % 3 mag.curves
    akap=[2]; ab=[.9:-.1:.2 .14 .1 .07 .05 .03 .02]; p1=.9;    % GOOD
            % ab=[.9 .5 .2 .1 .05];                    
    lkap=length(akap);  lb=length(ab);      % length of arrays akap, ab
    aba=zeros(lkap,lb); am=aba; af=am; ia=zeros(1,lkap)+lb; % 2D fields
%    
%%%%%%%%%%%%%%%%%% loop for many b=B/Bc2 values for magnetization curve            
  for ib=1:lb, b=ab(ib); nx=min(30+fix(4/sqrt(b)),100); % for small b
% for ib=1:lb, b=ab(ib); nx=20+fix(1.5/sqrt(b)),        % faster example
                                    % nx grid points along x, ny along y
  nx=nx; x1=1;  y2=x1*sqrt(3)/2;  x2=.5;     % triangular vortex lattice
% nx=nx; x1=1;  y2=1;             x2=0;      % square vortex lattice
  ny=fix(.5*nx*y2/x1+.5); n=nx*ny; yh=y2/2;  % egdes: (0,0) (x1,0) (x2,y2)
  x=x1*([.5:nx]/nx -.5); y=.5*y2*[.5:ny]/ny; yl=[-fliplr(y) y]; % grid x,y
  xx=[1:n]; yy=xx; xx(:)=ones(ny,1)*x; yy(:)=y'*ones(1,nx); f=0; f1=0;
  plot(xx,yy,'yo', [-x1 -x2 0 x2 x1], [0 y2 0 y2 0],'w-'); % plots 2D grid
  axis equal; axis([-x1/2 x1/2 -0*y2/2 y2/2]);
title('Real space: rectangular basic cell and grid points','fontsize',16);
                                                            pause(p1);
  pm=fix(0.5*nx); qm=fix(pm*sqrt(x2^2+y2^2)/x1 +1.01);    % k-vectors for
  km=pm*2*pi/x1; km2=km*km+eps; ck=2*pi/(x1*y2);          % Fourier series
        ik=0; kx=zeros(1,2); ky=kx; k2=kx; pq=kx;      % k=(kx,ky), k2=k^2
  for p=-pm:pm, for q=0:qm, lx=-p*y2*ck; ly=(q*x1+p*x2)*ck; % take only k
  ll=lx*lx+ly*ly; if ll<=km2 & (p>0 | q>0), ik=ik+1;        % in a circle
  kx(ik)=lx; ky(ik)=ly; k2(ik)=ll; pq(ik)=p+p*q+q; end; end; end; nk=ik;
          [k2 i]=sort(k2); kx=kx(i); ky=ky(i); pq=pq(i); % sort size of k
  b_nxny_n_nk_pmqm=[b nx ny n/100 nk/100 pm qm], kx2=kx./k2; ky2=ky./k2;
        p=[.5:200]*pi/100; k=sqrt(k2);        % plot grid of the (kx,ky)
        plot(kx,ky,'r.',-kx,-ky,'g+',km*cos(p),km*sin(p),'w'); axis equal;
  title('Reciprocal space: k-vector grid with k < km=kmax','fontsize',16);
                                                            pause(p1);
  % tabulate functions cx(x), cy(y), ckr(x,y), skr(x,y) for Fourier series
  xp=[-.4:.01:1.0]; yp=xp; cx=cos(kx'*xp); cy=cos(ky'*yp);  % for 1D plots
  ckr=cos(kx'*xx+ky'*yy); skr=sin(kx'*xx+ky'*yy);           % for 2D omega
                   % Fourier coefficients of om=omega(x,y)=|psi|^2 are ak,
                   % as initial values we take Abrikosov Bc2-solution ak0:
  ak0=-2*cos(pi*pq).*exp(-k2*x1*y2/(8*pi)); sak0=sum(ak0);  % Abrikosov
  om0=ak0*(1-ckr); om=om0; ak=ak0; omxy=zeros(ny,nx); omxy(:)=om;
         plot(k, ak0, 'w*-');
  title('Four.coefficients of Abrikosov Bc2 solution omega');  pause(p1);
% contour(x,y,flipud(omxy),30,'-'); title('Abrik.omega(x,y)'); pause(p1); 
                          o0=[fliplr(omxy') omxy']; mesh(-o0);
  title('Abrikosov B_{c2} omega=|\psi(x,y)|^2','fontsize',18); pause(p1);
     contour(x,yl,o0',30,'-'); axis([-1.35*yh 1.35*yh -yh yh]);
  title('Contours of Abrikosov |\psi(x,y)|^2','fontsize',18);  pause(p1);
     plot(xp,ak0*(1-cx),'w',yp,ak0*(1-cy),'r.-');
  title('Sections of Abrikosov |\psi(x,y)|^2','fontsize',18);  pause(p1);
                     % gradient omega /omega =(qx,qy), q2=(grad.om/om)^2
  q0x=((ak0.*ky)*skr)./om0; q0y=-((ak0.*kx)*skr)./om0; q02=q0x.^2+q0y.^2;
  gom0=(((ak0.*kx)*skr).^2+((ak0.*ky)*skr).^2)./om0;   % =(grad.om)^2/om
  cb=4*pi/y2; gq0=q02-((ak0.*k2)*ckr)./om0; f=0;  % gq0(x,y)=const=cb !!!
            maxcb=max(abs(gq0-cb)),  % maxcb =!= 0, test of this identity
% ggq=omxy; ggq(:)=gq0-gq0(1,1); mesh([fliplr(ggq') ggq']); pause; % test
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% loop over kappa %%%%%%%%%%%%%%%%%%%%
 for ikap=1:lkap, kap=akap(ikap);  % many kappa; if curvature of Ba(b) <0
     if ib>4 & aba(ikap,ib-1)*(ab(ib-3)-ab(ib-2))... % replace Ba(B) =Bc1
              -aba(ikap,ib-2)*(ab(ib-3)-ab(ib-1))... % by a vertical line
              +aba(ikap,ib-3)*(ab(ib-2)-ab(ib-1)) <= eps, ia(ikap)=ib;
     aba(ikap,ib)=aba(ikap,ib-2); aba(ikap,ib-1)=aba(ikap,ib-2); else,
%
%===== important CONSTANTS, b=B/Bc2, c=xi/(2*x1), cl=(lambda/x1)^2 ======
   c=sqrt(b*y2/(8*pi)); cc=c*c; ak=ak0*(1-b); cl=b*kap*kap*y2/(2*pi);
     b0=cb*c; q2=q02*cc; h=0*q2; qx=h; qy=h; com=-1;  % b0 =induction <B>
   % tabulate exact Bc1(kappa) from an interpolation formula in EB PRB 68
  if ib*ikap==1, lk=log(akap); al=.49693+exp(-.41477-.775*lk-.1303*lk.^2); 
                 bc1=(lk+al)./(2*akap.^2); end;    
%
 for it=1:150, %======= Iteration of GL equations for omega and h =======
   if it>1, fom=om.*(om+com-1+q2)+gom*cc;      % ak,hk =Four.coefficients
   ak= (2/n)*(fom*ckr')./(k2*2*cc -com); end;  % iteration of omega(x,y)
   om=ak*(1-ckr); gx=(ak.*kx)*skr; gy=(ak.*ky)*skr; gom=(gx.^2+gy.^2)./om;
   ca=-sum(- om+q2.*om+cc*gom)/sum(om.^2); ak=ak*ca; om=om*ca;
                                           gx=gx*ca; gy=gy*ca; gom=gom*ca;
   f1=f; f=sum(-om+.5*om.^2+q2.*om+cc*gom+cl*h.*h)/n;fi(it)=f;%free energy
                                 s=sum(ak); si(it)=s; % average: s=<omega>
   if it>6, gr=-om*b0 +(s-om).*h - (gx.*qy-gy.*qx);
            hk=(2/n)*(gr*ckr')./(k2*cl +s); h=hk*ckr; % teration of H(x,y)
   qx=c*q0x-(hk.*ky2)*skr; qy=c*q0y+(hk.*kx2)*skr; q2=qx.^2+qy.^2; 
        hx=hk*cx+b0; hy=hk*cy+b0; ox=ak*(1-cx); oy=ak*(1-cy); %H and omega
   plot(xp,hx,'yo-',yp,hy,'r.-',xp,ox,'wo-',yp,oy,'g.-',xp,xp*0+b0,'b');
             else,                        % the first 6 steps keep H=const
   plot(xp,ak*(1-cx),'wo',yp,ak*(1-cy),'g.-'); end; axis normal;
title(['b=',num2str(b),' kap=',num2str(kap),'  it=',num2str(it),'  F=',...
  num2str(f),'  dF =',num2str(f-f1),'  <om> =',num2str(s)],'fontsize',14);
                        pause(.01);  %show iteration like movie, F =!= min
   if it>14 & abs(f1-f)<(1-b)*1e-13, break; end; %condition stop iteration 
                                     end; pause(p1); %it, end of iteration
%                                     
        if 2>1,                      % SWITCH THIS ON, shows some plots!
   plot(4:it,100*fi(4:it),'r*-');    % check convergence of iteration
   title('Convergence of free energy f(it)','fontsize',14); pause(p1);
   plot(log10(abs(fi(1:it-1) -fi(it))+eps),'y*-');
   title('log10 of free energy f-f_{end}', 'fontsize',14);  pause(p1);
     % compare the Four.coeff. hk with model by Hao+Clem JLTP18 (1975) 427
     fin=sqrt(1-b^4); w1=sqrt(1-b^4+cl*k2); w2=sqrt(2);  % compare the H_K
     xvl=(w2-.75/kap)*sqrt((1+b^4)*(2*b*(1-b)^2-1))/kap; %this numeric: hk 
   bk=2*b0*fin*besselh(1,xvl*w1)./w1/besselh(1,fin*xvl); %bk =H_K Hao+Clem
   hkL=2*b0./(1+k2*cl); %hkL =hk of London Theory, good if b<0.001, kap>10
   j=1:50; plot(j,hk(j),'yo-',j,bk(j),'w*-',j,hkL(j),'r*-');
  title('Four.coeff. H_K compared with London and Hao+Clem','fontsize',14); 
                                      pause(p1); end; % if 1>2 switch
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%======= New Plots of Omega(x,y), Fig.1,2, FLLK62H.m ======================
%  X=x1*([-nx+.5:nx*2]/nx -.5); Y=.5*y2*[-3*ny+.5:ny*3]/ny; % grids X,Y
%  X=[-.5:.04167:1.51]; Y=[-.5:.04167:1.29];
   NX=24; NY=round(NX*y2); X=[-.5:1/NX:1.51]; Y=[-.5:y2/NY:1.29];%grids X,Y
          Nx=length(X); Ny=length(Y); N=Nx*Ny;  % NX=30
   M=NX*NY, Nh=NX/2; X1=X(Nh+[1:NX]); Y1=Y(Nh+[1:NY]);
%  XM=[1:M]; YM=XM; XM(:)=ones(NY,1)*X1; YM(:)=Y1'*ones(1,NX);
   XX=[1:N]; YY=XX; XX(:)=ones(Ny,1)*X;  YY(:)= Y'*ones(1,Nx);
   l=0; il=0; for i=1:N,
        if XX(i)>-.001 & XX(i)<.999 &YY(i)>-.001 & YY(i)<.865,
        l=l+1; il(l)=i; XM(l)=XX(i); YM(l)=YY(i); end; end; l,
   plot(XX,YY,'y.',XM,YM,'ro',[-x1 -x1 x1 x1 -x1], [0 y2 y2 0 0],'w-');
axis equal; title('New grid points, larger area','fontsize',16); pause(.2);
        %plot(Y1, cos(ky(1:17)'*Y1),'.-'); pause;     % Test cos(y)
        %plot(X1, cos(kx(1:17)'*X1),'.-'); pause;     % Test cos(x)
   %cc=cos(kx'*XM+ky'*YM)*cos(kx'*XM +ky'*YM)'/M*2;   % Test orthogonality
        %O= ak*(1-cos(kx'*XX+ky'*YY)); sum(sum(cc))/NX; cc(1:4,1:4);
   %Ak=-O(il)*cos(kx'*XM+ky'*YM)'*2/M; Ak-ak;  pause; % Test Fourier coeff.
                          for i1=1:1,
   oxy=zeros(Ny,Nx); oxy(:)=ak*(1-cos(kx'*XX+ky'*YY));  % potential oxy
%                    oxy=oxy*(1-t^4);   colormap white;
   l=mesh(X,Y,oxy); xlabel('x','fontsize',18); ylabel('y','fontsize',18);
   axis([-.5 1.5 -.5 1.27 0 1]); zlabel('\omega(x,y)','fontsize',18);
            set(l,'linewidth',0.4); set(gca,'fontsize',14,'linewidth',1);
   title(['b = ',num2str(b),',    \kappa = ',num2str(kap)],'fontsize',16);
                 hgsave('fgA'); print -dps2 -append fgA; pause(.5);
%  contour(X,Y,oxy,20,'-'); axis equal; axis tight; % axis(1.02*axis);
%  title('Contours of |\psi(x,y)|^2','fontsize',16);

   bxy=zeros(Ny,Nx); bxy(:)=hk*cos(kx'*XX+ky'*YY)+b0;   % magnetic field
                     bc2=b0/b; bxy=bxy/bc2;      % unit of bxy is now Bc2  
   l=mesh(X,Y,bxy); xlabel('x','fontsize',18); ylabel('y','fontsize',18);
                    zlabel('B(x,y) / B_{c2}','fontsize',18);
   axis([-.5 1.5 -.5 1.27 0 max(max(bxy))]); set(l,'linewidth',0.4);
   title(['b = ',num2str(b),',    \kappa = ',num2str(kap)],'fontsize',16);
   set(gca,'fontsize',14,'linewidth',1); %title('Fig.2','fontsize',14);
   hgsave('fgB'); print -dps2 -append fgB; pause(.5); end; end; % i, kap
     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                                  
   ba=sum(om.*(1-om) +2*cl*(h+b0).^2)/(n*2*b*kap^2); % Applied Field H=ba
             af(ikap,ib)=f; aba(ikap,ib)=ba; end; % from: if b>.7 
                     am=aba-ones(lkap,1)*ab; end; % ikap; am=magnetization
                                      % end kappa loop =================== 
   l=plot([1;aba';bc1],[0;am';bc1],'w.-',[0 1],[0 1],'b--',bc1,bc1,'ro');
     xlabel('applied field   B_a / B_{c2}',          'fontsize',18); 
     ylabel('negative magnetization  - M / B_{c2}','fontsize',18);
     set(l,'linewidth',.9,'markersize',8); 
     set(gca,'fontsize',14,'linewidth',1.1);
     text(0.1,0.88,'magnetization curve','fontsize',18);
     text(0.1,0.8,['for   \kappa = ',num2str(kap)],'fontsize',18);
                               print -dps2 -append fgB; pause(.5);
   save data05x.mat aba am ab akap lkap lb nx ia n nk af bc1; 
   %vit% end; % ib
   %vit% end; pause; % break;             % end b=B/Bc2 loop =================
   
   % after computation show magnetization curves several times  
   for ii=1:6, 
      pause(1); axis([0 .25 0 .25]); 
      pause(1); axis([0 .06 0 .06]); 
      pause(1); axis([0 .016 0 .016]); 
      pause(1); axis([0 1 0 1]); 
      pause(1); 
   end;
     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% THE END %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
